@@ -15,6 +15,7 @@ import inspect
 import os
 import platform
 import sys
+import sysconfig
 import textwrap
 import warnings
 import zipfile
@@ -143,7 +144,7 @@ def run_tests_with_coverage(tracer, *runner_args):
     # The .pth file seems to have to be alphabetically after easy-install.pth
     # or the sys.path entries aren't created right?
     # There's an entry in "make clean" to get rid of this file.
-    pth_dir = os.path.dirname(pytest.__file__)
+    pth_dir = sysconfig.get_path("purelib")
     pth_path = os.path.join(pth_dir, "zzz_metacov.pth")
     with open(pth_path, "w") as pth_file:
         pth_file.write("import coverage; coverage.process_startup()\n")
@@ -233,7 +234,7 @@ def do_zip_mods():
             """)
         # These encodings should match the list in tests/test_python.py
         details = [
-            ('utf8', 'ⓗⓔⓛⓛⓞ, ⓦⓞⓡⓛⓓ'),
+            ('utf-8', 'ⓗⓔⓛⓛⓞ, ⓦⓞⓡⓛⓓ'),
             ('gb2312', '你好，世界'),
             ('hebrew', 'שלום, עולם'),
             ('shift_jis', 'こんにちは世界'),
