@@ -93,8 +93,8 @@ def test_flat_rootname(original, flat):
 
 
 @pytest.mark.parametrize(
-        "patterns, case_insensitive, partial,"
-            "matches,"
+        "patterns, case_insensitive, partial," +
+            "matches," +
             "nomatches",
 [
     (
@@ -172,7 +172,7 @@ class MatcherTest(CoverageTest):
             files.canonical_filename("SUB5/file6.py"),
             ]
         tm = TreeMatcher(trees, "test")
-        assert tm.info() == trees
+        assert tm.info() == sorted(trees)
         for filepath, matches in matches_to_try:
             self.assertMatches(tm, filepath, matches)
 
