@@ -130,7 +130,7 @@ class ProcessTest(CoverageTest):
         self.assert_exists(".coverage")
         self.assert_exists(".coverage.bad")
         warning_regex = (
-            r"CoverageWarning: Couldn't use data file '.*\.coverage\.bad': "
+            r"CoverageWarning: Couldn't use data file '.*\.coverage\.bad': " +
             r"file (is encrypted or )?is not a database"
         )
         assert re.search(warning_regex, out)
@@ -163,9 +163,8 @@ class ProcessTest(CoverageTest):
         for n in "12":
             self.assert_exists(f".coverage.bad{n}")
             warning_regex = (
-                r"CoverageWarning: Couldn't use data file '.*\.coverage.bad{}': "
+                fr"CoverageWarning: Couldn't use data file '.*\.coverage.bad{n}': " +
                 r"file (is encrypted or )?is not a database"
-                .format(n)
             )
             assert re.search(warning_regex, out)
         assert re.search(r"No usable data files", out)
@@ -469,8 +468,11 @@ class ProcessTest(CoverageTest):
 
     def test_code_throws(self):
         self.make_file("throw.py", """\
+            class MyException(Exception):
+                pass
+
             def f1():
-                raise Exception("hey!")
+                raise MyException("hey!")
 
             def f2():
                 f1()
@@ -489,9 +491,9 @@ class ProcessTest(CoverageTest):
 
         # But also make sure that the output is what we expect.
         path = python_reported_file('throw.py')
-        msg = f'File "{re.escape(path)}", line 5,? in f2'
+        msg = f'File "{re.escape(path)}", line 8, in f2'
         assert re.search(msg, out)
-        assert 'raise Exception("hey!")' in out
+        assert 'raise MyException("hey!")' in out
         assert status == 1
 
     def test_code_exits(self):
@@ -725,9 +727,9 @@ class ProcessTest(CoverageTest):
         assert "Goodbye!" in out
 
         msg = (
-            "CoverageWarning: "
-            "Already imported a file that will be measured: {} "
-            "(already-imported)").format(goodbye_path)
+            f"CoverageWarning: Already imported a file that will be measured: {goodbye_path} " +
+            "(already-imported)"
+        )
         assert msg in out
 
     @pytest.mark.expensive
@@ -759,7 +761,7 @@ class ProcessTest(CoverageTest):
     # Pypy passes locally, but fails in CI? Perhaps the version of macOS is
     # significant?  https://foss.heptapod.net/pypy/pypy/-/issues/3074
     @pytest.mark.skipif(env.PYPY, reason="PyPy is unreliable with this test")
-    # Jython as of 2.7.1rc3 won't compile a filename that isn't utf8.
+    # Jython as of 2.7.1rc3 won't compile a filename that isn't utf-8.
     @pytest.mark.skipif(env.JYTHON, reason="Jython can't handle this test")
     def test_lang_c(self):
         # LANG=C forces getfilesystemencoding on Linux to 'ascii', which causes
@@ -818,7 +820,6 @@ class ProcessTest(CoverageTest):
         # Remove the file location and source line from the warning.
         out = re.sub(r"(?m)^[\\/\w.:~_-]+:\d+: CoverageWarning: ", "f:d: CoverageWarning: ", out)
         out = re.sub(r"(?m)^\s+self.warn.*$\n", "", out)
-        print("out:", repr(out))
         expected = (
             "Run 1\n" +
             "Run 2\n" +
@@ -1123,7 +1124,7 @@ class ExcepthookTest(CoverageTest):
         assert cov_st == py_st
         assert cov_st == 0
 
-        assert "in excepthook" in py_out
+        assert py_out == "in excepthook\n"
         assert cov_out == py_out
 
     @pytest.mark.skipif(env.PYPY, reason="PyPy handles excepthook throws differently.")
@@ -1313,10 +1314,10 @@ class UnicodeFilePathsTest(CoverageTest):
         assert ' name="h\xe2t.py"'.encode() in xml
 
         report_expected = (
-            "Name     Stmts   Miss  Cover\n"
-            "----------------------------\n"
-            "h\xe2t.py       1      0   100%\n"
-            "----------------------------\n"
+            "Name     Stmts   Miss  Cover\n" +
+            "----------------------------\n" +
+            "h\xe2t.py       1      0   100%\n" +
+            "----------------------------\n" +
             "TOTAL        1      0   100%\n"
         )
 
@@ -1357,12 +1358,12 @@ class UnicodeFilePathsTest(CoverageTest):
         }
 
         report_expected = (
-            "Name            Stmts   Miss  Cover\n"
-            "-----------------------------------\n"
-            "\xe2%saccented.py       1      0   100%%\n"
-            "-----------------------------------\n"
-            "TOTAL               1      0   100%%\n"
-        ) % os.sep
+            "Name            Stmts   Miss  Cover\n" +
+            "-----------------------------------\n" +
+            f"\xe2{os.sep}accented.py       1      0   100%\n" +
+            "-----------------------------------\n" +
+            "TOTAL               1      0   100%\n"
+        )
 
         out = self.run_command("coverage report")
         assert out == report_expected
@@ -1411,7 +1412,7 @@ def possible_pth_dirs():
 
     # If we're still looking, then try the Python library directory.
     # https://github.com/nedbat/coveragepy/issues/339
-    yield sysconfig.get_python_lib()                # pragma: cant happen
+    yield sysconfig.get_path("purelib")             # pragma: cant happen
 
 
 def find_writable_pth_directory():
@@ -1616,7 +1617,6 @@ class ProcessStartupWithSourceTest(ProcessCoverageMixin, CoverageTest):
         data = coverage.CoverageData()
         data.read()
         summary = line_counts(data)
-        print(summary)
         assert summary[source + '.py'] == 3
         assert len(summary) == 1
 
@@ -1655,9 +1655,9 @@ def run_in_venv(cmd):
     """
     words = cmd.split()
     if env.WINDOWS:
-        words[0] = r"{}\Scripts\{}.exe".format("venv", words[0])
+        words[0] = fr"venv\Scripts\{words[0]}.exe"
     else:
-        words[0] = "{}/bin/{}".format("venv", words[0])
+        words[0] = fr"venv/bin/{words[0]}"
     status, output = run_command(" ".join(words))
     assert status == 0
     return output
@@ -1694,13 +1694,37 @@ def venv_world_fixture(tmp_path_factory):
             def fourth(x):
                 return 4 * x
             """)
+        # Some namespace packages.
+        make_file("third_pkg/nspkg/fifth/__init__.py", """\
+            def fifth(x):
+                return 5 * x
+            """)
+        # The setup.py to install everything.
         make_file("third_pkg/setup.py", """\
             import setuptools
-            setuptools.setup(name="third", packages=["third", "fourth"])
+            setuptools.setup(
+                name="third",
+                packages=["third", "fourth", "nspkg.fifth"],
+            )
+            """)
+
+        # Some namespace packages.
+        make_file("another_pkg/nspkg/sixth/__init__.py", """\
+            def sixth(x):
+                return 6 * x
+            """)
+        # The setup.py to install everything.
+        make_file("another_pkg/setup.py", """\
+            import setuptools
+            setuptools.setup(
+                name="another",
+                packages=["nspkg.sixth"],
+            )
             """)
 
         # Install the third-party packages.
         run_in_venv("python -m pip install --no-index ./third_pkg")
+        run_in_venv("python -m pip install --no-index -e ./another_pkg")
         shutil.rmtree("third_pkg")
 
         # Install coverage.
@@ -1722,6 +1746,8 @@ def coverage_command_fixture(request):
 class VirtualenvTest(CoverageTest):
     """Tests of virtualenv considerations."""
 
+    expected_stdout = "33\n110\n198\n1.5\n"
+
     @pytest.fixture(autouse=True)
     def in_venv_world_fixture(self, venv_world):
         """For running tests inside venv_world, and cleaning up made files."""
@@ -1729,10 +1755,13 @@ class VirtualenvTest(CoverageTest):
             self.make_file("myproduct.py", """\
                 import colorsys
                 import third
+                import nspkg.fifth
+                import nspkg.sixth
                 print(third.third(11))
+                print(nspkg.fifth.fifth(22))
+                print(nspkg.sixth.sixth(33))
                 print(sum(colorsys.rgb_to_hls(1, 0, 0)))
                 """)
-            self.expected_stdout = "33\n1.5\n"      # pylint: disable=attribute-defined-outside-init
 
             self.del_environ("COVERAGE_TESTING")    # To avoid needing contracts installed.
             self.set_environ("COVERAGE_DEBUG_FILE", "debug_out.txt")
@@ -1741,7 +1770,7 @@ class VirtualenvTest(CoverageTest):
             yield
 
             for fname in os.listdir("."):
-                if fname != "venv":
+                if fname not in {"venv", "another_pkg"}:
                     os.remove(fname)
 
     def get_trace_output(self):
@@ -1786,14 +1815,13 @@ class VirtualenvTest(CoverageTest):
             r"^Not tracing .*\bexecfile.py': " +
             "module 'coverage.execfile' falls outside the --source spec"
             )
-        print(re_lines(debug_out, "myproduct"))
         assert re_lines(
             debug_out,
-            r"^Not tracing .*\bmyproduct.py': module u?'myproduct' falls outside the --source spec"
+            r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec"
             )
         assert re_lines(
             debug_out,
-            r"^Not tracing .*\bcolorsys.py': module u?'colorsys' falls outside the --source spec"
+            r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec"
             )
 
         out = run_in_venv("python -m coverage report")
@@ -1833,3 +1861,45 @@ class VirtualenvTest(CoverageTest):
         # The output should not have this warning:
         # Already imported a file that will be measured: ...third/render.py (already-imported)
         assert out == "HTML: hello.html@1723\n"
+
+    def test_installed_namespace_packages(self, coverage_command):
+        # https://github.com/nedbat/coveragepy/issues/1231
+        # When namespace packages were installed, they were considered
+        # third-party packages.  Test that isn't still happening.
+        out = run_in_venv(coverage_command + " run --source=nspkg myproduct.py")
+        # In particular, this warning doesn't appear:
+        # Already imported a file that will be measured: .../coverage/__main__.py
+        assert out == self.expected_stdout
+
+        # Check that our tracing was accurate. Files are mentioned because
+        # --source refers to a file.
+        debug_out = self.get_trace_output()
+        assert re_lines(
+            debug_out,
+            r"^Not tracing .*\bexecfile.py': " +
+            "module 'coverage.execfile' falls outside the --source spec"
+            )
+        assert re_lines(
+            debug_out,
+            r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec"
+            )
+        assert re_lines(
+            debug_out,
+            r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec"
+            )
+
+        out = run_in_venv("python -m coverage report")
+
+        # Name                                                       Stmts   Miss  Cover
+        # ------------------------------------------------------------------------------
+        # another_pkg/nspkg/sixth/__init__.py                            2      0   100%
+        # venv/lib/python3.9/site-packages/nspkg/fifth/__init__.py       2      0   100%
+        # ------------------------------------------------------------------------------
+        # TOTAL                                                          4      0   100%
+
+        assert "myproduct.py" not in out
+        assert "third" not in out
+        assert "coverage" not in out
+        assert "colorsys" not in out
+        assert "fifth" in out
+        assert "sixth" in out
